/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* JprobeInfoDialog.java,v 1.4 2002/08/02 15:39:23 jba Exp */
/**
 * This is the PV Info Dialog
 *
 * @author <A HREF="mailto:jba@aps.anl.gov">
 * Janet Anderson</A>
 *
 * @version 1.0
*/

package jprobe;

import javax.swing.*;
import javax.swing.border.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import jca.*;
import jca.event.*;
import jca.dbr.*;

public class JprobeInfoDialog extends JDialog implements ValueEventListener
{
  protected JPanel   historyPanel = null;
  protected JPanel   infoPanel = null;
  protected JprobePv jprobePv;
  protected JPanel   mainPanel = null;
  protected double   maximum = 0;
  protected Date     maximumTime = null;
  protected double   minimum = 0;
  protected Date     minimumTime = null;
  protected Date     monitorStartTime = null;
  protected Date     monitorStartTimePrev = null;
  protected Font     monoFont;
  protected Date     prevValueStringDate = null;
  protected String   prevValueString = null;
  protected String   pvName = null;

  protected JLabel nameLabel,fieldTypeLabel,unitsLabel;
  protected JLabel precisionLabel,elementCountLabel;
  protected JLabel hostLabel,readAccessLabel,writeAccessLabel;
  protected JLabel hoprLabel,loprLabel,drvhLabel,drvlLabel;
  protected JLabel hihiLabel,loloLabel,highLabel,lowLabel;
  protected JLabel startTimeLabel,maximumLabel,minimumLabel;
  protected JLabel valueLabel,statusLabel,severityLabel;


  JprobeInfoDialog(JFrame parent,JprobePv jPv)
  {
    super(parent);
    monoFont = new Font("monospaced",Font.BOLD,12);

    jprobePv = jPv;
    pvName = jPv.getName();
    setTitle("Jprobe: Info Dialog");
    getContentPane().setLayout(new BorderLayout(5,5));

    mainPanel = new JPanel();
    mainPanel.setLayout(new BorderLayout(0,0));
    createInfoPanel();
    mainPanel.add(infoPanel,BorderLayout.NORTH);

    createHistoryPanel();
    mainPanel.add(historyPanel,BorderLayout.SOUTH);
    getContentPane().add(mainPanel,BorderLayout.NORTH);

    JPanel buttonPanel = new JPanel();
    JButton closeButton = new JButton("Close");
    Border raisedBorder = new BevelBorder(BevelBorder.RAISED);
    Border emptyBorder = new EmptyBorder(0,5,0,5);
    closeButton.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    closeButton.addActionListener( new ActionListener() {
           public void actionPerformed(ActionEvent e)
           { JprobeInfoDialog.this.dispose(); } });
    buttonPanel.add(closeButton);
    buttonPanel.setBorder(new EtchedBorder());
    getContentPane().add(buttonPanel,BorderLayout.SOUTH);

    updateDialog();

    pack();
    setLocation(100,100);
    //setSize(200,310);
    setVisible(true);

    jprobePv.addValueEventListener(this);
  }

  protected void pvNameChanged(JprobePv jpv)
  {
    pvName = jpv.getName();
    monitorStartTime = null;
    monitorStartTimePrev = null;
    mainPanel.removeAll();
    createHistoryPanel();
    mainPanel.add(historyPanel,BorderLayout.SOUTH);
    createInfoPanel();
    mainPanel.add(infoPanel,BorderLayout.NORTH);
    mainPanel.revalidate();
    pack();
  }

  public void valueChanged(ValueEvent e) {
      jprobePv = e.jprobePv();
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        updateDialog();
      }
    });
  }

  protected void updateDialog()
  {
    JprobePv pv = jprobePv;

    if (!pv.getName().equals(pvName)) {
    	pvNameChanged(pv);
	return;
    }
    if (pv.isMonitored()){
      if (monitorStartTime == null){
        monitorStartTime=Calendar.getInstance().getTime();
        Class dbrClass = pv.getDBR().getClass();
        if ((dbrClass == DBR_CTRL_DBFloat.class ||
             dbrClass == DBR_CTRL_DBDouble.class ||
             dbrClass == DBR_CTRL_DBShort.class ||
             dbrClass == DBR_CTRL_DBInt.class)) {
          maximum = pv.getDBR().dataAt(0).doubleValue();
          minimum = maximum;
        } else {
          prevValueString = pv.getValueString();
          prevValueStringDate = Calendar.getInstance().getTime();
        }
      } else {
        if (pv.getDBR().dataAt(0).doubleValue()>maximum) {
          maximum = pv.getDBR().dataAt(0).doubleValue();
          maximumTime = Calendar.getInstance().getTime();
        }
        if (pv.getDBR().dataAt(0).doubleValue()<minimum) {
          minimum = pv.getDBR().dataAt(0).doubleValue();
          minimumTime = Calendar.getInstance().getTime();
        }
      }
    } else {
      monitorStartTime = null;
      monitorStartTimePrev = null;
      maximum = 0;
      minimum = 0;
    }
    //updateInfoPanel(pv.getDBR());
    updateHistoryPanel(pv.getDBR());
  }

  protected void createInfoPanel()
  {
    infoPanel = new JPanel();

    infoPanel.setLayout(new GridLayout(0,1));
    Border infoBorder = new TitledBorder(null, "PV's Information",
              TitledBorder.LEFT,TitledBorder.TOP);
    Border emptyBorder = new EmptyBorder(0,5,0,5);
    infoPanel.setBorder(new CompoundBorder(infoBorder,emptyBorder));

    nameLabel = new JLabel("PV Name        : ");
    nameLabel.setFont(monoFont);
    infoPanel.add(nameLabel);
    fieldTypeLabel = new JLabel("Field Type     : ");
    fieldTypeLabel.setFont(monoFont);
    infoPanel.add(fieldTypeLabel);
    if (jprobePv.getDBR() != null){
      Class dbrClass = jprobePv.getDBR().getClass();
      if (jprobePv.getDBR() != null &&
         (dbrClass == DBR_CTRL_DBFloat.class ||
	  dbrClass == DBR_CTRL_DBDouble.class))
      {
        unitsLabel = new JLabel("Units          : ");
        unitsLabel.setFont(monoFont);
        infoPanel.add(unitsLabel);
        precisionLabel = new JLabel("Precision      : ");
        precisionLabel.setFont(monoFont);
        infoPanel.add(precisionLabel);
      }
    }
    elementCountLabel = new JLabel("Element Count  : ");
    elementCountLabel.setFont(monoFont);
    infoPanel.add(elementCountLabel);
    hostLabel = new JLabel("Host Name      : ");
    hostLabel.setFont(monoFont);
    infoPanel.add(hostLabel);
    readAccessLabel = new JLabel("Read Access    : ");
    readAccessLabel.setFont(monoFont);
    infoPanel.add(readAccessLabel);
    writeAccessLabel = new JLabel("Write Access   : ");
    writeAccessLabel.setFont(monoFont);
    infoPanel.add(writeAccessLabel);

    if (jprobePv.getDBR() == null) return;
    Class dbrClass = jprobePv.getDBR().getClass();
    if (dbrClass == DBR_CTRL_DBFloat.class ||
        dbrClass == DBR_CTRL_DBDouble.class ||
        dbrClass == DBR_CTRL_DBInt.class ||
        dbrClass == DBR_CTRL_DBShort.class) {
      hoprLabel = new JLabel();
      hoprLabel.setFont(monoFont);
      infoPanel.add(hoprLabel);
      loprLabel = new JLabel();
      loprLabel.setFont(monoFont);
      infoPanel.add(loprLabel);
      drvhLabel = new JLabel();
      drvhLabel.setFont(monoFont);
      infoPanel.add(drvhLabel);
      drvlLabel = new JLabel();
      drvlLabel.setFont(monoFont);
      infoPanel.add(drvlLabel);
      hihiLabel = new JLabel();
      hihiLabel.setFont(monoFont);
      infoPanel.add(hihiLabel);
      highLabel = new JLabel();
      highLabel.setFont(monoFont);
      infoPanel.add(highLabel);
      lowLabel = new JLabel();
      lowLabel.setFont(monoFont);
      infoPanel.add(lowLabel);
      loloLabel = new JLabel();
      loloLabel.setFont(monoFont);
      infoPanel.add(loloLabel);
    }

    updateInfoPanel(jprobePv.getDBR());
    return;
  }

  protected void createHistoryPanel()
  {
    infoPanel = new JPanel();
    historyPanel = new JPanel();
    historyPanel.setLayout(new GridLayout(0,1));
    Border historyBorder = new TitledBorder(null, "History",
              TitledBorder.LEFT,TitledBorder.TOP);
    Border emptyBorder = new EmptyBorder(0,5,0,5);
    historyPanel.setBorder(new CompoundBorder(historyBorder,emptyBorder));

    startTimeLabel = new JLabel("Start Time    :                                 ");
    startTimeLabel.setFont(monoFont);
    historyPanel.add(startTimeLabel);
    maximumLabel = new JLabel("Maximum value : ");
    maximumLabel.setFont(monoFont);
    historyPanel.add(maximumLabel);
    minimumLabel = new JLabel("Minimum value : ");
    minimumLabel.setFont(monoFont);
    historyPanel.add(minimumLabel);

    updateHistoryPanel(jprobePv.getDBR());
    return;
  }

  protected void updateInfoPanel(DBR dbr)
  {
    if (infoPanel == null ) return;
    if (dbr == null ) return;

    nameLabel.setText("PV Name        : " +
      jprobePv.getPV().name());
    fieldTypeLabel.setText("Field Type     : " + 
      getFieldTypeString(jprobePv.getPV().fieldType()));
    elementCountLabel.setText("Element Count  : " +
      jprobePv.getPV().elementCount());
    hostLabel.setText("Host Name      : " +
      jprobePv.getPV().hostName());
    readAccessLabel.setText("Read Access    : " +
      jprobePv.getPV().readAccess());
    writeAccessLabel.setText("Write Access   : " +
      jprobePv.getPV().writeAccess());

    Class dbrClass = dbr.getClass();

    if (jprobePv.getFormat() != null &&
       (dbrClass == DBR_CTRL_DBFloat.class ||
        dbrClass == DBR_CTRL_DBDouble.class))
    {
      DBR_CTRL ctrl = (DBR_CTRL)dbr;
      if (dbrClass == DBR_CTRL_DBDouble.class) {
        precisionLabel.setText("Precision      : "
          + ((DBR_CTRL_DBDouble)ctrl).precision());
      }
      if (dbrClass == DBR_CTRL_DBFloat.class) {
        precisionLabel.setText("Precision      : "
          + ((DBR_CTRL_DBFloat)ctrl).precision());
      }
      unitsLabel.setText(    "Units          : " + ctrl.units());
      hoprLabel.setText("Upper display limit (HOPR) : " +
        jprobePv.getFormat().format(ctrl.upperDispLimit().doubleValue()));
      loprLabel.setText("Lower display limit (LOPR) : " +
        jprobePv.getFormat().format(ctrl.lowerDispLimit().doubleValue()));
      drvhLabel.setText("Upper control limit (DRVH) : " +
        jprobePv.getFormat().format(ctrl.upperCtrlLimit().doubleValue()));
      drvlLabel.setText("Lower control limit (DRVL) : " +
        jprobePv.getFormat().format(ctrl.lowerCtrlLimit().doubleValue()));
      hihiLabel.setText("Upper alarm limit   (HIHI) : " +
        jprobePv.getFormat().format(ctrl.upperAlarmLimit().doubleValue()));
      highLabel.setText("Upper warning limit (HIGH) : " +
        jprobePv.getFormat().format(ctrl.upperWarningLimit().doubleValue()));
      lowLabel.setText("Lower warning limit (LOW)  : " +
        jprobePv.getFormat().format(ctrl.lowerWarningLimit().doubleValue()));
      loloLabel.setText("Lower alarm limit   (LOLO) : " +
        jprobePv.getFormat().format(ctrl.lowerAlarmLimit().doubleValue()));

    }
    else if (dbrClass == DBR_CTRL_DBInt.class ||
             dbrClass == DBR_CTRL_DBShort.class ||
             dbrClass == DBR_CTRL_DBFloat.class ||
             dbrClass == DBR_CTRL_DBDouble.class) {
      DBR_CTRL ctrl = (DBR_CTRL)dbr;
      if (dbrClass == DBR_CTRL_DBDouble.class) {
        precisionLabel.setText("Precision      : "
		+ ((DBR_CTRL_DBDouble)ctrl).precision());
        unitsLabel.setText(    "Units          : " + ctrl.units());
      }
      if (dbrClass == DBR_CTRL_DBFloat.class) {
        precisionLabel.setText("Precision      : "
		+ ((DBR_CTRL_DBFloat)ctrl).precision());
        unitsLabel.setText(    "Units          : " + ctrl.units());
      }
      hoprLabel.setText("Upper display limit (HOPR) : " +
                        ctrl.upperDispLimit().stringValue());
      loprLabel.setText("Lower display limit (LOPR) : " +
                        ctrl.lowerDispLimit().stringValue());
      drvhLabel.setText("Upper control limit (DRVH) : " +
                        ctrl.upperCtrlLimit().stringValue());
      drvlLabel.setText("Lower control limit (DRVL) : " +
                        ctrl.lowerCtrlLimit().stringValue());
      hihiLabel.setText("Upper alarm limit   (HIHI) : " +
                        ctrl.upperAlarmLimit().stringValue());
      highLabel.setText("Upper warning limit (HIGH) : " +
                        ctrl.upperWarningLimit().stringValue());
      lowLabel.setText("Lower warning limit (LOW)  : " +
                        ctrl.lowerWarningLimit().stringValue());
      loloLabel.setText("Lower alarm limit   (LOLO) : " +
                        ctrl.lowerAlarmLimit().stringValue());
    }
  }

  protected void updateHistoryPanel(DBR dbr)
  {
    if (historyPanel == null ) return;

    if (monitorStartTime != monitorStartTimePrev ) {
      startTimeLabel.setText("Start Time    : " + monitorStartTime);
      monitorStartTimePrev = monitorStartTime;
    }

    if (dbr == null ) return;
    Class dbrClass = dbr.getClass();

    if (dbrClass == DBR_STS_DBString.class) {
      if (monitorStartTime == null) {
        startTimeLabel.setText("Start Time    : ");
        maximumLabel.setText("Last String : ");
        minimumLabel.setText(" ");
      } else {
        maximumLabel.setText("Last String   : " + jprobePv.getValueString());
        minimumLabel.setText(" ");
      }
      return;
    }

   if (dbrClass == DBR_INFO_DBEnum.class) {
     if (monitorStartTime == null) {
       startTimeLabel.setText("Start Time    : ");
       maximumLabel.setText("Last State : ");
       minimumLabel.setText(" ");
     } else {
       maximumLabel.setText("Last State    : " + jprobePv.getValueString());
       minimumLabel.setText(" ");
     }
     return;
   }

    //if (monitorStartTime == null) {
//        startTimeLabel.setText("Start Time    : ");
//        maximumLabel.setText("Maximum value : ");
//        minimumLabel.setText("Minimum value : ");
//	    return;
//    }
    if (dbrClass == DBR_CTRL_DBFloat.class ||
        dbrClass == DBR_CTRL_DBDouble.class )
    {
      if (jprobePv.getFormat() != null ) {
        maximumLabel.setText("Maximum value : " +
          jprobePv.getFormat().format(maximum));
        minimumLabel.setText("Minimum value : " +
          jprobePv.getFormat().format(minimum));
      } else {
        maximumLabel.setText("Maximum value : " + maximum);
        minimumLabel.setText("Minimum value : " + minimum);
      }
	  return;
    }

    if (dbrClass == DBR_CTRL_DBInt.class ||
        dbrClass == DBR_CTRL_DBShort.class) {
      maximumLabel.setText("Maximum value : " + maximum);
      minimumLabel.setText("Minimum value : " + minimum);
	  return;
    }
  }

  protected String getFieldTypeString(int type)
  {
    switch (type) {
    case DBR.STRING:
      return "String";
    case DBR.SHORT:
      return "Short";
    case DBR.FLOAT:
      return "Float";
    case DBR.ENUM:
      return "Enum";
    case DBR.BYTE:
      return "Char";
    case DBR.INT:
      return "Integer";
    case DBR.DOUBLE:
      return "Double";
    default:
      return "Unknown";
    }
  }
}

